//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Base/Axis/DiscreteAxis.h
//! @brief     Defines class DiscreteAxis.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_BASE_AXIS_DISCRETEAXIS_H
#define BORNAGAIN_BASE_AXIS_DISCRETEAXIS_H

#include "Base/Axis/IAxis.h"

//! Axis containing discrete data points for use in scans.

class DiscreteAxis : public IAxis {
public:
    DiscreteAxis(const std::string& name, const std::vector<double>& coordinate_values);
    DiscreteAxis(const std::string& name, size_t n, double first, double last);

    //! clone function
    DiscreteAxis* clone() const override;

    //! retrieve the number of bins
    size_t size() const override { return m_coordinates.size(); }

    //! retrieve a 1d bin for the given index
    Bin1D bin(size_t index) const override;

    //! Returns value of first on-axis point
    double min() const override;

    //! Returns value of last on-axis point
    double max() const override;

    //! Returns the coordinate corresponding to the
    //! given index. For this type of axis ("pointwise")
    //! this equals returning an explicitly defined coordinate.
    double binCenter(size_t index) const override;

    //! find index of the coordinate closest to the given value
    size_t findClosestIndex(double value) const override;

    std::vector<double> binCenters() const override { return m_coordinates; }

    std::vector<double> binBoundaries() const override;

    void clip(double lower, double upper) override;

private:
    void print(std::ostream& ostr) const override;
    bool equals(const IAxis& other) const override;

    double minary(size_t index) const;
    double maxary(size_t index) const;
    void checkIndex(size_t index) const;
    void sanityCheck() const;

    std::vector<double> m_coordinates;
};

#endif // BORNAGAIN_BASE_AXIS_DISCRETEAXIS_H
