//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Multilayer/MultilayerUtil.cpp
//! @brief     Implements helper functions for MultiLayer objects.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Multilayer/MultilayerUtil.h"
#include "Base/Util/Assert.h"
#include "Sample/Interface/LayerInterface.h"
#include "Sample/Multilayer/Layer.h"
#include "Sample/Multilayer/MultiLayer.h"

const LayerRoughness* SampleUtil::Multilayer::LayerTopRoughness(const MultiLayer& sample, size_t i)
{
    if (i == 0)
        return nullptr;
    return sample.layerInterface(i - 1)->roughness();
}

size_t SampleUtil::Multilayer::IndexOfLayer(const MultiLayer& sample, const Layer* p_layer)
{
    for (size_t i = 0; i < sample.numberOfLayers(); ++i)
        if (p_layer == sample.layer(i))
            return i;
    ASSERT(false);
}

bool SampleUtil::Multilayer::hasRoughness(const MultiLayer& sample)
{
    for (size_t i = 0; i < sample.numberOfLayers() - 1; i++)
        if (sample.layerInterface(i)->roughness())
            return true;
    return false;
}
